/*================DESKRIPTION===============
 The program reads a input image in PGM format.
 It allocates memory to store the pixel data 
 of the input image. The "reading_image" function 
 is used to create a new image where pixel 
 values above a threshold become 255,
 and those below become 0. The program 
 prints the pixel values of the first and 
 last pixels in the thresholded image.
 The "writer" function is used to write 
 the thresholded image to a new PGM file
 Memory allocated for pixel data is freed
 to prevent memory leaks. 
 ===========================================*/


#include <stdio.h>
#include <stdlib.h>


typedef struct
{
	int w; 		 // width of the image
	int h; 		 // height of the image
	int *pixels; // the pixels of th imgage
} image;

//function to "tröskla" an image based on the given "trösel" value
image reading_image(const image *input, int reading)
{
	image result;
	result.w = input ->  w;
	result.h = input ->  h;
	
	//allocats memory for pixel data  	
	result.pixels = (int *)malloc(result.w * result.h * sizeof(int));


	//"trösklig" pixle value 
	for (int i = 0; i < result.w * result.h; i++)
	{
		result.pixels[i] = (input -> pixels[i] >= reading) ? 255 : 0;
	}
	return result;
}

//function to write a image to a PMG-file 
void writer(const char *filename, const image *img)
{
FILE *file = fopen(filename, "w");

if(file == NULL)
{
	perror("generic error message1");
	exit(EXIT_FAILURE);
}

//write PMG header	
fprintf(file,"P2\n %d %d\n255\n",img -> w, img -> h);


//write pixel value 
for(int i = 0; i < img->w * img->h; i++)
{
	 fprintf(file, "%d ", img->pixels[i]);
}

fclose(file);
	}
	
int main(int argc , char *argv[])
{
	if(argc != 2)
	{
		printf("generic error message2");
		return EXIT_FAILURE;
	}	
	
	
	//reads the inputed image 
	image input; 
	FILE *input_file = fopen(argv[1],"r");
	if(input_file == NULL)
	{
		perror("generic error mesage3");
		return EXIT_FAILURE;
	}
	fscanf(input_file, "P2 %d %d 255", &input.w, &input.h); 
	input.pixels = (int *)malloc(input.w * input.h * sizeof(int));
	
	for(int i = 0; i < input.w * input.h;i++)
	{
		fscanf(input_file, "%d", &input.pixels[i]);
	}
	fclose(input_file);
	//"trösklar" the image
	int reading_value = 120;
	image read_image = reading_image(&input, reading_value);
	//prints the first and last pixle of the "tröskald" image 
	printf("%d %d\n",read_image.pixels[0], read_image.pixels[input.w * input.h - 1]);
	printf("jobs done\n");// a message to confirm the good work
	
	// Write the "trösklade" image to a new file
	writer("threshold.pgm",&read_image);
	
	// Free allocated memory
	free(input.pixels);
	free(read_image.pixels);
	
	return EXIT_SUCCESS;
}






